from jinja2 import Environment, FileSystemLoader

def generate_file():
  file_loader = FileSystemLoader('src/huble/sklearn/templates')
  env = Environment(loader=file_loader)
  template = env.get_template('script.j2')
  output = template.render(name='Rugved')
  print(output)




def return_function(node):
    parameters = node["parameters"]
    if node["data"]["value"] == "Normalization":
        return normalize_template(parameters)
    elif node["data"]["value"] == "Drop Duplicates":
        return drop_duplicates_template(parameters)


def normalize_template(params):
    """
    Template for normalization function
    """
    # TODO: Convert node data to normalize function parameters

    return f"data = huble.sklearn.normalize(column_name='{params['column_name']}', data=data)"


def fillna_template(params):
    """
    Template for fillna function
    """
    pass


def drop_duplicates_template(params):
    """
    Template for drop_duplicates function
    """
    return f"huble.sklearn.drop_duplicates(data_frame=data,subset={params['subset']},keep=\"{params['keep']}\")"


from ast import parse
import pandas as pd


def parse_dataset(dataset):
    dict = {}
    dict['rows'] = len(dataset)
    dict['columns'] = []
    for column in dataset.columns:
        if column.lower() == 'location' or column.lower() == 'city' or column.lower() == 'country' or column.lower() == 'state':
            dict['columns'].append({'name': column, 'type': 'location'})
        if column in dataset.select_dtypes(include=['int64', 'float64', 'int32', 'float32']).columns.values:
            dict['columns'].append({'name': column, 'type': 'numeric'})
        elif column in dataset.select_dtypes(include=['object']).columns.values:
            if len(dataset[column].unique()) < len(dataset)*0.1:
                dict['columns'].append({'name': column, 'type': 'categorical'})
                continue
            try:
                pd.to_datetime(dataset[column], infer_datetime_format=True)
                dict['columns'].append({'name': column, 'type': 'datetime'})
                continue
            except:
                dict['columns'].append({'name': column, 'type': 'text'})



from re import sub
from sklearn import preprocessing
import pandas as pd


def normalize(params):
    preprocessing.normalize(**params)


def drop_duplicates(data_frame=None, subset=None, keep="first"):
    #TODO : Add support for ignore index
    data_frame.drop_duplicates(subset=subset, keep=keep)




from scipy import stats
from xgboost import XGBRegressor
import numpy as np
import pandas as pd


def verify_ml_dataset(dataset):
    # check for null values in pandas dataframe
    DATA_IMBALANCE_THRESHOLD = 0.1
    TOO_MANY_ZEROES_THRESHOLD = 0.1
    FEATURE_IMPORTANCE_THRESHOLD = 0.01

    checks = {"null_values": "",
              "too_many_zeroes": "",
              "label": "",
              "outliers": "",
              "data_imbalance": "",
              "data_format": "",
              "low_importance_features": "",
              "data_normalization": "",
              "string_features": ""
              }
    Y = pd.DataFrame()

    # check for null values in pandas dataframe
    if dataset.isnull().values.any():
        checks["null_values"] = "error"
    else:
        checks["null_values"] = "ok"

    # check for label config
    for column in dataset.columns:
        if 'label' in column:
            Y[column] = dataset[column]
            dataset.drop(column, axis=1, inplace=True)
    if Y.empty:
        checks["label"] = "error"
    else:
        checks["label"] = "ok"

    # check for string columns in pandas dataframe
    if (dataset.dtypes == object).sum() > 0:
        checks["string_features"] = "error"
    else:
        checks["string_features"] = "ok"

    # check for too many zeroes in pandas dataframe
    count = 0
    for column_name in dataset.columns:
        column = dataset[column_name]
        count += (column == 0).sum()
        if count > len(dataset)*TOO_MANY_ZEROES_THRESHOLD:
            checks["too_many_zeroes"] = "warning"

    # Check for outliers
    if checks['string_features'] == "ok":
        z_scores = stats.zscore(dataset)
        abs_z_scores = np.abs(z_scores)
        # Select data points with a z-scores above or below 3
        filtered_entries = (abs_z_scores < 3).all(axis=1)
        if len(filtered_entries) > 0:
            checks["outliers"] = "warning"
        else:
            checks["outliers"] = "ok"

    # Check for data imbalance
    if checks['label'] == 'ok':
        try:
            values = Y.value_counts()
            if values[0]*DATA_IMBALANCE_THRESHOLD > values[1] or values[1]*DATA_IMBALANCE_THRESHOLD > values[0]:
                checks["data_imbalance"] = "warning"
            else:
                checks["data_imbalance"] = "ok"
        except:
            labels = []
            for column_name in Y.columns:
                if 'label' in column_name:
                    labels.append(dataset[column_name][1])
            labels.sort()
            if labels[0]*DATA_IMBALANCE_THRESHOLD > labels[-1] or labels[-1]*DATA_IMBALANCE_THRESHOLD > labels[0]:
                checks["data_imbalance"] = "warning"
            else:
                checks["data_imbalance"] = "ok"

    # check for feature importance
    if checks['string_features'] == "ok":
        xgb = XGBRegressor()
        xgb.fit(dataset, Y)
        importance = xgb.feature_importances_
        for score in importance:
            if score < FEATURE_IMPORTANCE_THRESHOLD:
                checks["low_importance_features"] = "warning"
                break
        else:
            checks["low_importance_features"] = "ok"

    # Data normalization
    if checks['string_features'] == "ok":
        for val in dataset.min():
            if val < -1:
                checks["data_normalization"] = "warning"
                break
        else:
            checks["data_normalization"] = "ok"
        for val in dataset.max():
            if val > 1:
                checks["data_normalization"] = "warning"
                break
        else:
            checks["data_normalization"] = "ok"

    return checks



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['huble',
 'huble.sklearn',
 'huble.sklearn.automl',
 'huble.sklearn.deploy',
 'huble.sklearn.process',
 'huble.sklearn.train',
 'huble.util']

package_data = \
{'': ['*'], 'huble.sklearn': ['templates/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'black>=22.8.0,<23.0.0',
 'pandas>=1.4.4,<2.0.0',
 'scikit-learn>=1.1.2,<2.0.0']

setup_kwargs = {
    'name': 'huble',
    'version': '0.1.21',
    'description': '',
    'long_description': '',
    'author': 'Rugz007',
    'author_email': 'rugvedsomwanshi007@gmail.com',
    'maintainer': 'Rugz007',
    'maintainer_email': 'rugvedsomwanshi007@gmail.com',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
